# Pico:ed V2 Buzzer, LED and graphics demonstration - Circuitpython
# Tony Goodhew 28th September 2022
# Instructables Tutorial
import board         # Gets the names of the pins used by CircuitPython
import time
import digitalio     # Library for digital Input Output
from picoed import * # Includes music to play the notes

'''
Importing picoed stops the normal pwmio method being
used on the Buzzer pin to produce tones.

If you try you will get the error message
   ValueError: BUZZER in use
   
To produce sounds we have to use elecfreaks_music library
Documentation: https://github.com/elecfreaks/circuitpython_ef_music
'''

# Beethoven’s 5th Symphony
notes = ['r4:2', 'g', 'g', 'g', 'eb:8', 'r:2', 'f', 'f', 'f', 'd:8']

music.play(notes)

#Graphics start here

W = display.width
H = display.height

def blit(s,x,y,w,h,v): # Insert image in frame
    for xx in range(w):
        for yy in range(h):
            c = s[xx+yy*w]*v
            display.pixel(x+xx,y+yy,c)

# Titles
display.scroll("Tony Goodhew")
display.scroll("Graphics Demo of Pico:ed V2")
#Brightness
display.fill(1) # Fill display with dimmest pixels
time.sleep(2)
display.fill(10) # Fill display with brighter pixels
time.sleep(2)

delay2 = 0.1
# Lines
display.fill(0) # clear the screen to BLACK
for i in range(7):  # Vertical line in column 0
    display.pixel(0,i, 10) # pixel ON
    time.sleep(delay2)
for i in range(7):  
    display.pixel(0,i, 0) # pixel OFF 
    time.sleep(delay2)
for i in range(17): # Horizontal line
    display.pixel(i,3, 10) # pixel ON
    time.sleep(delay2)
for i in range(17):
    display.pixel(i,3, 0) # pixel OFF
    time.sleep(delay2)

# Single pixel getting brighter
for i in range(255):
    display.pixel(3,3, i) 
    time.sleep(0.01)

# Line of pixels getting brighter
step = int(255/16)
for i in range(i):
    display.pixel(i,5, i * step)
display.pixel(16,5, 255)
time.sleep(2.5)
display.fill(0)

# Pretty edge - Some pixels off the screen without error
c = 10 # colour/brightness
for i in range(17):
    display.pixel(i,0, c)
    display.pixel(17-i,6, c)
    display.pixel(0,i, c)  
    display.pixel(16,7-i, c) 
    time.sleep(0.1)
time.sleep(1)

# Gradient screen fill
c = 1
inc = 1
for x in range(17):
    for y in range(7):
        display.pixel(x,y,c)
    c = c + inc
time.sleep(3)

# Moving diagonal line - alternating frames
frame = 0
display.frame(frame, show=False)
for x in range(-7, 18,1):
    display.frame(frame, show=False)
    display.fill(0)    
    for xx in range(7):
        for yy in range(7):
            display.pixel(xx+x,xx,10)
    display.frame(frame, show=True)
    time.sleep(0.2)
    if frame == 0: # switch frames
        frame = 1
    else:
        frame = 0
            
# 3x3 squares
display.sleep(True)
display.fill(0)
for y in range(0,H,2):
    for x in range(W):
        display.pixel(x,y,10)
for x in range(0,W,2):
    for y in range(H):
        display.pixel(x,y,10)        
display.sleep(False)
time.sleep(2)

# Display an image 
shades = [1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
          1,1,1,1,1,1,1,0,0,0,1,1,1,1,1,1,1,
          0,1,1,1,1,1,0,0,0,0,0,1,1,1,1,1,0,
          0,0,1,1,1,0,0,0,0,0,0,0,1,1,1,0,0,
          0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
          0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,
          0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,0] # Each 1/0 is a pixel - 1 is pixel ON & 0 is pixel OFF
   
display.sleep(True)
for y in range(H):
    for x in range(W):
        display.pixel(x, y, shades[x + y * W]*20)
display.sleep(False)
                   
time.sleep(3)

# Pixel Bounce
display.show("Pixel Bounce")
display.show("Press A to Halt")
dx = 1
dy = 1
running = True
x = 3
y = 3
while running:
    display.pixel(x,y, 10) # Current pixel ON
    time.sleep(.05)
    oldx = x
    oldy = y
    x = x + dx
    if x  == 17:
        x = 15
        dx = -1
    if x == -1:
        x = 1
        dx = 1
    y = y + dy
    if y  == 7:
        y = 5
        dy = -1
    if y == -1:
        y = 1
        dy = 1
    display.pixel(oldx,oldy, 0) # Turn old pixel OFF
    # Press button A to stop loop
    if button_a.is_pressed(): 
        running = False
display.fill(0)

# Built in images for information
''' Built in Image names
    NO, SQUARE, RECTANGLE, RHOMBUS, TARGET, CHESSBOARD, HAPPY, SAD, YES, HEART, TRIANGLE, CHAGRIN, SMILING_FACE 
    CRY, DOWNCAST, LOOK_RIGHT, LOOK_LEFT, TONGUE, PEEK_RIGHT, PEEK_LEFT, TEAR_EYES, PROUD 
    SNEER_LEFT, SNEER_RIGHT, SUPERCILIOUS_LOOK, EXCITED 
'''
display.show(Image.SAD)
time.sleep(1.5)
display.show(Image.HAPPY)
time.sleep(1.5)

# Image blitting
heart = [0,1,1,0,1,1,0, # 7 wide by 6 high
         1,1,1,1,1,1,1,
         1,1,1,1,1,1,1,
         0,1,1,1,1,1,0,
         0,0,1,1,1,0,0,
         0,0,0,1,0,0,0]

display.sleep(True)
display.fill(0)
blit(heart,5,0,7,6,10) # Display stationary heart
display.sleep(False)
time.sleep(2)

display.sleep(True)
display.fill(0)
display.sleep(False)

# Currently without frames or sleeep - May show random flashes on screen
#frame = 0
for p in range(17,-7,-1): # Heart moving left to right
    #display.sleep(True)
    #display.frame(frame, show=False)
    display.fill(0)           # clear screen
    blit(heart,p,0,7,6,30)    # Draw heart on screen
    #display.sleep(False)
    #display.frame(frame, show=True)
    time.sleep(0.15)

#  ==== Duck sequence with random flashing suppressed ====
duckL = [0,1,0,0,0,0, # 6x6
         1,1,1,0,0,0,
         0,1,1,0,0,0,
         0,1,0,0,0,1,
         1,1,1,1,1,1,
         0,1,1,1,1,0]

duckR = [0,0,0,0,1,0, # 6x6
         0,0,0,1,1,1,
         0,0,0,1,1,0,
         1,0,0,0,1,0,
         1,1,1,1,1,1,
         0,1,1,1,1,0]

display.sleep(True) # Screen update OFF
display.fill(0)
blit(duckL,6,0,6,6,10) # Display stationary duck - left facing
display.sleep(False) # Screen update ON
time.sleep(2)

display.sleep(True)
display.fill(0)
blit(duckR,6,0,6,6,10) # Display stationary duck - right facing
display.sleep(False)
time.sleep(2)

display.sleep(True)
display.fill(0)
display.sleep(False)
frame = 0
for p in range(17,-7,-1): # Swimming duck R -> L
    #display.sleep(True)
    display.frame(frame, show=False) # Works better to suppress random flashes on screen
    display.fill(0)
    blit(duckL,p,0,6,6,30)
    #display.sleep(False)
    display.frame(frame, show=True) # Works better
    time.sleep(0.15)
    if frame == 0:
        frame = 1
    else:
        frame = 0
    
display.sleep(True)
display.fill(0)
display.sleep(False)
frame = 0
for p in range(-6,23,1): # Swimming duck L -> R
    #display.sleep(True)
    display.frame(frame, show=False)
    display.fill(0)
    blit(duckR,p,0,6,6,30)
    #display.sleep(False)
    display.frame(frame, show=True)
    time.sleep(0.15)
    if frame == 0:
        frame = 1
    else:
        frame = 0

display.sleep(True)
display.fill(0)
display.sleep(False)

# LED Flashing
display.show("LED")
led_0 = digitalio.DigitalInOut(board.LED)
led_0.direction = digitalio.Direction.OUTPUT
for i in range(6):
    led_0.value = True
    time.sleep(0.3)
    led_0.value = False
    time.sleep(0.3)

# Finish
display.show("Bye")
time.sleep(2)
display.fill(0) # Clear display


    